require_dependency 'search_controller'

class SearchController
  def software_infos
    prepare_software_search_page
    results = filter_software_infos_list
    @software_count = results.count
    results = results.paginate(:per_page => @per_page, :page => params[:page])
    @searches[@asset] = {:results => results}
    @search = results

    render :layout=>false if request.xhr?
  end

  protected

  def filter_software_infos_list
    filtered_software_info_list = get_filtered_software_info_list
    filtered_software_list = get_softwares_list(filtered_software_info_list)
    sort_softwares_list filtered_software_list
  end

  def get_filter_category_ids
    category_ids = []
    unless params[:selected_categories_id].blank?
      category_ids = params[:selected_categories_id]
    end
    category_ids.map(&:to_i)
  end

  def get_filtered_software_info_list
    params[:query] ||= ""
    visible_software = visible_profiles(Software)

    filtered_software_info_list = SoftwareInfo.search_by_query(params[:query])

    if params[:only_softwares]
      params[:only_softwares].collect!{ |software_name| software_name.to_slug }
      filtered_software_info_list = SoftwareInfo.all.select{ |s| params[:only_softwares].include?(s.identifier) }
      @public_software_selected = false
    end

    filtered_software_info_list.select!{ |software_info| visible_software.include?(software_info.software) }
    category_ids = get_filter_category_ids

    unless category_ids.empty?
      filtered_software_info_list.select! do |software_info|
        if software_info.nil? || software_info.software.nil?
          false
        else
          result_ids = (software_info.software.category_ids & category_ids).sort
          result_ids == category_ids.sort
        end
      end
    end

    filtered_software_info_list
  end

  def get_softwares_list software_info_list
    filtered_software_list = []
      software_info_list.each do |software_info|
       if !@public_software_selected || software_info.public_software?
         filtered_software_list << software_info.software unless software_info.software.nil?
       end
    end
    filtered_software_list
  end

  def sort_softwares_list softwares_list
    softwares_list.sort! {|a, b| a.name.downcase <=> b.name.downcase}

    if params[:sort] && params[:sort] == "desc"
      softwares_list.reverse!
    elsif params[:sort] && params[:sort] == "relevance"
      softwares_list = sort_by_relevance(softwares_list, params[:query]){ |software| [software.software_info.finality, software.name] }
    end
    softwares_list
  end

  def prepare_software_search_page
    prepare_software_infos_params
    prepare_software_infos_message
    prepare_software_infos_category_groups
    prepare_software_infos_category_enable
  end

  def prepare_software_infos_params
    @titles[:software_infos] = _("Result Search")
    @selected_categories_id = params[:selected_categories_id]
    @selected_categories_id ||= []
    @selected_categories_id = @selected_categories_id.map(&:to_i)
    @all_selected = params[:software_type] == "all"
    @public_software_selected = !@all_selected
    @per_page = prepare_per_page
  end

  def prepare_per_page
    return 15 if params[:software_display].nil?

    if params[:software_display] == "all"
      SoftwareInfo.count
    else
      params[:software_display].to_i
    end
  end

  def prepare_software_infos_message
    @message_selected_options = ""

    @selected_categories = []
    unless @selected_categories_id.empty?
      @message_selected_options = _("Selected options: ")

      @selected_categories = Category.find(@selected_categories_id)
      @message_selected_options += @selected_categories.collect { |category|
        "#{category.name}; "
      }.join()
    end
  end

  def prepare_software_infos_category_groups
    @categories = Category.software_categories.sort{|a, b| a.name <=> b.name}
  end

  def prepare_software_infos_category_enable
    @enabled_check_box = Hash.new
    categories = Category.software_categories

    categories.each do |category|
      if category.softwares.count > 0
        @enabled_check_box[category] = :enabled
      else
        @enabled_check_box[category] = :disabled
      end
    end
  end
end
